@@ -1,7 +1,7 @@
 
 from .memory import MemoryRepo
 from .mongo import MongoRepo
-from .sqlalchemy import SQLRepo
+from .sqlalchemy import SQLRepo, SQLExprRepo
 from .rest import RESTRepo
 from .csv import CSVFileRepo
 from .json import JSONDirectoryRepo
\ No newline at end of file
