@@ -12,7 +12,7 @@ import mongomock
 from redbird.repos.csv import CSVFileRepo
 from redbird.repos.json import JSONDirectoryRepo
 from redbird.repos.rest import RESTRepo
-from redbird.repos.sqlalchemy import SQLRepo
+from redbird.repos.sqlalchemy import SQLExprRepo, SQLRepo
 from redbird.repos.memory import MemoryRepo
 from redbird.repos.mongo import MongoRepo
 from redbird.oper import greater_equal, greater_than, less_equal, less_than, not_equal
@@ -42,6 +42,13 @@ class MongoItem(BaseModel):
     name: str
     age: int
 
+class PydanticItemWithDefaults(BaseModel):
+    __id_field__ = 'id'
+    id: str
+    name: str
+    age: Optional[int] = 20
+    color: str = Field(default_factory=lambda: "black")
+
 try:
     from sqlalchemy import Column, String, Integer, create_engine
     from sqlalchemy.orm import declarative_base
@@ -180,40 +187,38 @@ class RESTMock:
         )
 
 
-def get_repo(type_, tmpdir):
+def get_repo(type_, tmpdir, model=PydanticItem):
     if type_.startswith("sql-"):
         pytest.importorskip("sqlalchemy")
         from sqlalchemy import Column, String, Integer, create_engine
+        import sqlalchemy
     if type_ == "memory":
-        repo = MemoryRepo(model=PydanticItem, id_field="id")
+        repo = MemoryRepo(model=model, id_field="id")
 
     elif type_ == "memory-dict":
         repo = MemoryRepo(model=dict, id_field="id")
 
     elif type_ == "csv":
-        repo = CSVFileRepo(model=PydanticItem, id_field="id", filename=str(tmpdir / "repo.csv"))
+        repo = CSVFileRepo(model=model, id_field="id", filename=str(tmpdir / "repo.csv"))
         repo.create()
 
     elif type_ == "json-dir":
-        repo = JSONDirectoryRepo(model=PydanticItem, id_field="id", path=str(tmpdir))
+        repo = JSONDirectoryRepo(model=model, id_field="id", path=str(tmpdir))
 
     elif type_ == "sql-dict":
         engine = create_engine('sqlite://')
-        engine.execute("""CREATE TABLE pytest (
-            id TEXT PRIMARY KEY,
-            name TEXT,
-            age INTEGER
-        )""")
+        with engine.begin() as conn:
+            conn.execute(sqlalchemy.text("""
+            CREATE TABLE pytest (
+                id TEXT PRIMARY KEY,
+                name TEXT,
+                age INTEGER
+            )"""))
         repo = SQLRepo(engine=engine, table="pytest", id_field="id")
 
     elif type_ == "sql-pydantic":
         engine = create_engine('sqlite://')
-        engine.execute("""CREATE TABLE pytest (
-            id TEXT PRIMARY KEY,
-            name TEXT,
-            age INTEGER
-        )""")
-        repo = SQLRepo(model=PydanticItem, engine=engine, table="pytest", id_field="id")
+        repo = SQLRepo(model=model, engine=engine, table="pytest", id_field="id", if_missing='create')
         #SQLItem.__table__.create(bind=repo.session.bind)
 
     elif type_ == "sql-orm":
@@ -221,6 +226,11 @@ def get_repo(type_, tmpdir):
         repo = SQLRepo(model_orm=SQLItem, engine=engine, table="items", id_field="id")
         repo.create()
 
+    elif type_ == "sql-expr":
+        engine = create_engine('sqlite://')
+        repo = SQLExprRepo(model=model, engine=engine, table="items", id_field="id")
+        repo.create()
+
     elif type_ == "sql-pydantic-orm":
         engine = create_engine('sqlite://')
         repo = SQLRepo(model=PydanticItemORM, model_orm=SQLItem, engine=engine, id_field="id")
@@ -228,11 +238,11 @@ def get_repo(type_, tmpdir):
 
     elif type_ == "mongo-mock":
         pytest.importorskip("pymongo")
-        repo = MongoRepo(model=PydanticItem, uri="mongodb://localhost:27017/pytest?authSource=admin", database="pytest", collection="items", id_field="id")
+        repo = MongoRepo(model=model, uri="mongodb://localhost:27017/pytest?authSource=admin", database="pytest", collection="items", id_field="id")
  
     elif type_ == "mongo":
         pytest.importorskip("pymongo")
-        repo = MongoRepo(model=PydanticItem, uri=get_mongo_uri(), database="pytest", collection="items", id_field="id")
+        repo = MongoRepo(model=model, uri=get_mongo_uri(), database="pytest", collection="items", id_field="id")
 
         # Empty the collection
         import pymongo
@@ -244,7 +254,7 @@ def get_repo(type_, tmpdir):
         col.delete_many({})
 
     elif type_ == "http-rest":
-        repo = RESTRepo(model=PydanticItem, url="http://localhost:5000/api/items", id_field="id")
+        repo = RESTRepo(model=model, url="http://localhost:5000/api/items", id_field="id")
 
     return repo
 
@@ -255,6 +265,20 @@ def get_repo(type_, tmpdir):
 @pytest.fixture
 def repo(request, tmpdir):
     repo = get_repo(request.param, tmpdir)
+    if request.param == "http-rest":
+        api = RESTMock()
+        with responses.RequestsMock(assert_all_requests_are_fired=False) as rsps:
+            api.add_routes(rsps)
+            yield repo
+    elif request.param == "mongo-mock":
+        with mongomock.patch(servers=(('localhost', 27017),)):
+            yield repo
+    else:
+        yield repo
+
+@pytest.fixture
+def repo_defaults(request, tmpdir):
+    repo = get_repo(request.param, tmpdir, model=PydanticItemWithDefaults)
     if request.param == "http-rest":
         api = RESTMock()
         with responses.RequestsMock(assert_all_requests_are_fired=False) as rsps:
