@@ -25,6 +25,7 @@ TEST_CASES = [
     pytest.param("sql-dict"),
     pytest.param("sql-pydantic"),
     pytest.param("sql-orm"),
+    pytest.param("sql-expr"),
     pytest.param("sql-pydantic-orm"),
     pytest.param("mongo"),
     pytest.param("mongo-mock"),
@@ -33,6 +34,16 @@ TEST_CASES = [
     pytest.param("json-dir"),
 ]
 
+TEST_CASES_DEFAULT = [
+    pytest.param("memory"),
+    pytest.param("sql-pydantic"),
+    pytest.param("sql-expr"),
+    pytest.param("mongo"),
+    pytest.param("mongo-mock"),
+    pytest.param("http-rest"),
+    pytest.param("csv"),
+    pytest.param("json-dir"),
+]
 
 def sort_items(items, repo, field="id"):
     return list(sorted(items, key=lambda x: repo.get_field_value(x, field)))
@@ -749,4 +760,20 @@ class TestMalformedData(RepoTests):
         if not repo.ordered:
             actual = sort_items(actual, repo)
             expected = sort_items(expected, repo)
-        assert actual == expected
\ No newline at end of file
+        assert actual == expected
+
+@pytest.mark.parametrize(
+    'repo_defaults',
+    TEST_CASES_DEFAULT,
+    indirect=True
+)
+class TestItemDefaults(RepoTests):
+
+    def test_add_with_default(self, repo_defaults):
+        repo = repo_defaults
+        Item = repo.model
+        
+        assert repo.filter_by().all() == []
+
+        repo.add(Item(id="a", name="Jack"))
+        assert repo.filter_by().all() == [Item(id="a", name="Jack", age=20, color="black")]
\ No newline at end of file
