@@ -1,9 +1,16 @@
 
 import datetime
+import sys
+import typing
 import pytest
 from redbird.repos import SQLRepo
 from pydantic import BaseModel
 
+try:
+    from typing import Literal
+except ImportError:
+    from typing_extensions import Literal
+
 class MyItem(BaseModel):
     id: str
     name: str
@@ -35,15 +42,23 @@ else:
                 return False
             return other.id == self.id and other.name == self.name and other.age == self.age
 
+def execute_sql(text, engine):
+    import sqlalchemy
+    with engine.begin() as conn:
+        return conn.execute(sqlalchemy.text(text))
+
 def test_init_engine():
     pytest.importorskip("sqlalchemy")
     from sqlalchemy import create_engine
     engine = create_engine('sqlite://')
-    engine.execute("""CREATE TABLE pytest (
-        id TEXT PRIMARY KEY,
-        name TEXT,
-        age INTEGER
-    )""")
+    execute_sql(
+        """CREATE TABLE pytest (
+            id TEXT PRIMARY KEY,
+            name TEXT,
+            age INTEGER
+        )""",
+        engine=engine
+    )
     repo = SQLRepo(engine=engine, table="pytest")
     assert repo.model_orm.__table__.name == "pytest"
     assert all(hasattr(repo.model_orm, col) for col in ("id", "name", "age"))
@@ -54,14 +69,18 @@ def test_init_engine():
 def test_init_conn_string(tmpdir):
     pytest.importorskip("sqlalchemy")
     from sqlalchemy import create_engine
+    import sqlalchemy
     with tmpdir.as_cwd() as old_dir:
         conn_string = 'sqlite:///pytest.db'
         engine = create_engine(conn_string)
-        engine.execute("""CREATE TABLE pytest (
-            id TEXT PRIMARY KEY,
-            name TEXT,
-            age INTEGER
-        )""")
+        execute_sql(
+            """CREATE TABLE pytest (
+                id TEXT PRIMARY KEY,
+                name TEXT,
+                age INTEGER
+            )""",
+            engine=engine
+        )
         repo = SQLRepo(conn_string=conn_string, table="pytest")
         assert repo.model_orm.__table__.name == "pytest"
         assert all(hasattr(repo.model_orm, col) for col in ("id", "name", "age"))
@@ -75,11 +94,11 @@ def test_init_session():
     from sqlalchemy.orm import Session
     engine = create_engine('sqlite://')
     session = Session(engine)
-    engine.execute("""CREATE TABLE pytest (
+    execute_sql("""CREATE TABLE pytest (
         id TEXT PRIMARY KEY,
         name TEXT,
         age INTEGER
-    )""")
+    )""", engine=engine)
     repo = SQLRepo(session=session, table="pytest")
     assert repo.model_orm.__table__.name == "pytest"
     assert all(hasattr(repo.model_orm, col) for col in ("id", "name", "age"))
@@ -91,11 +110,11 @@ def test_init_model():
     pytest.importorskip("sqlalchemy")
     from sqlalchemy import create_engine
     engine = create_engine('sqlite://')
-    engine.execute("""CREATE TABLE pytest (
+    execute_sql("""CREATE TABLE pytest (
         id TEXT PRIMARY KEY,
         name TEXT,
         age INTEGER
-    )""")
+    )""", engine=engine)
     repo = SQLRepo(model=MyItem, engine=engine, table="pytest")
     assert repo.model is MyItem
 
@@ -106,11 +125,11 @@ def test_init_model_orm_mode():
     pytest.importorskip("sqlalchemy")
     from sqlalchemy import create_engine
     engine = create_engine('sqlite://')
-    engine.execute("""CREATE TABLE pytest (
+    execute_sql("""CREATE TABLE pytest (
         id TEXT PRIMARY KEY,
         name TEXT,
         age INTEGER
-    )""")
+    )""", engine=engine)
     repo = SQLRepo(model=MyItemWithORM, engine=engine, table="pytest")
     assert repo.model is MyItemWithORM
 
@@ -121,11 +140,11 @@ def test_init_orm():
     pytest.importorskip("sqlalchemy")
     from sqlalchemy import create_engine
     engine = create_engine('sqlite://')
-    engine.execute("""CREATE TABLE pytest (
+    execute_sql("""CREATE TABLE pytest (
         id TEXT PRIMARY KEY,
         name TEXT,
         age INTEGER
-    )""")
+    )""", engine=engine)
     repo = SQLRepo(model_orm=SQLItem, reflect_model=True, engine=engine)
     assert repo.model_orm is SQLItem
     assert issubclass(repo.model, BaseModel)
@@ -137,11 +156,11 @@ def test_init_model_and_orm():
     pytest.importorskip("sqlalchemy")
     from sqlalchemy import create_engine
     engine = create_engine('sqlite://')
-    engine.execute("""CREATE TABLE pytest (
+    execute_sql("""CREATE TABLE pytest (
         id TEXT PRIMARY KEY,
         name TEXT,
         age INTEGER
-    )""")
+    )""", engine=engine)
     repo = SQLRepo(model=MyItem, model_orm=SQLItem, engine=engine)
     assert repo.model_orm is SQLItem
     assert issubclass(repo.model, BaseModel)
@@ -153,11 +172,11 @@ def test_init_dict():
     pytest.importorskip("sqlalchemy")
     from sqlalchemy import create_engine
     engine = create_engine('sqlite://')
-    engine.execute("""CREATE TABLE pytest (
+    execute_sql("""CREATE TABLE pytest (
         id TEXT PRIMARY KEY,
         name TEXT,
         age INTEGER
-    )""")
+    )""", engine=engine)
 
     repo = SQLRepo(model=dict, engine=engine, table="pytest")
     repo.add({"id": "a", "name": "Jack", "age": 500})
@@ -173,17 +192,19 @@ def test_init_deprecated(tmpdir):
         pytest.importorskip("sqlalchemy")
         from sqlalchemy import create_engine
         engine = create_engine('sqlite:///pytest.db')
-        engine.execute("""CREATE TABLE pytest (
+        execute_sql("""CREATE TABLE pytest (
             id TEXT PRIMARY KEY,
             name TEXT,
             age INTEGER
-        )""")
+        )""", engine=engine)
 
-        repo = SQLRepo.from_engine(model=dict, engine=engine, table="pytest")
+        with pytest.warns(DeprecationWarning):
+            repo = SQLRepo.from_engine(model=dict, engine=engine, table="pytest")
         repo.add({"id": "a", "name": "Jack", "age": 500})
         assert list(repo) == [{"id": "a", "name": "Jack", "age": 500}]
         
-        repo = SQLRepo.from_connection_string(model=dict, conn_string="sqlite:///pytest.db", table="pytest")
+        with pytest.warns(DeprecationWarning):
+            repo = SQLRepo.from_connection_string(model=dict, conn_string="sqlite:///pytest.db", table="pytest")
         repo.add({"id": "b", "name": "Jack", "age": 5000})
         assert list(repo) == [{"id": "a", "name": "Jack", "age": 500}, {"id": "b", "name": "Jack", "age": 5000}]
 
@@ -196,11 +217,11 @@ def test_init_reflect_model_without_primary_key():
     with pytest.raises(KeyError):
         repo = SQLRepo(model=MyItem, engine=engine, table="mytable", if_missing="create")
     
-    engine.execute("""CREATE TABLE pytest (
+    execute_sql("""CREATE TABLE pytest (
         id TEXT,
         name TEXT,
         age INTEGER
-    )""")
+    )""", engine=engine)
     with pytest.raises(KeyError):
         repo = SQLRepo(model=MyItem, engine=engine, table="mytable")
 
@@ -218,20 +239,32 @@ def test_init_reflect_model_id_field_in_model():
 
     # Make sure the __id_field__ is not in there
     repo.add(MyItem(name="Jack", age=20))
-    obs = engine.execute("select * from mytable")
+    obs = execute_sql("select * from mytable", engine=engine)
     assert list(obs) == [('Jack', 20)]
 
-def test_init_type():
+@pytest.mark.parametrize("cls,example_value", [
+    pytest.param(str, "Jack", id="str"),
+    pytest.param(int, 5, id="int"),
+    pytest.param(float, 5.2, id="float"),
+
+    pytest.param(datetime.date, datetime.date(2022, 1, 1), id="datetime.date"),
+    pytest.param(datetime.datetime, datetime.datetime(2022, 1, 1, 12, 30), id="datetime.datetime"),
+    pytest.param(datetime.timedelta, datetime.timedelta(days=2), id="datetime.timedelta", marks=pytest.mark.skip("SQLite does not support Interval")),
+
+    pytest.param(dict, {"name": "Jack"}, id="dict"),
+
+    pytest.param(typing.Optional[str], "Jack", id="Optional[str]"),
+    pytest.param(typing.Union[str, None], "Jack", id="Union[str, None]", marks=pytest.mark.skipif(sys.version_info < (3, 8), reason="<py38")),
+    pytest.param(Literal['yes', 'no'], "yes", id="Literal['...', '...']", marks=pytest.mark.skipif(sys.version_info < (3, 8), reason="<py38")),
+    pytest.param(Literal[1, 2], 2, id="Literal[1, 2]", marks=pytest.mark.skipif(sys.version_info < (3, 8), reason="<py38")),
+])
+def test_init_type_insert(cls, example_value):
     pytest.importorskip("sqlalchemy")
 
     class MyItem(BaseModel):
         __id_field__ = "id"
         id: str
-        name: str
-        age: int
-        birth_date: datetime.date
-        observed: datetime.datetime
-        meta: dict
+        myfield: cls
 
     from sqlalchemy import create_engine
     engine = create_engine('sqlite://')
@@ -239,24 +272,70 @@ def test_init_type():
     
     repo.add(MyItem(
         id="a",
-        name="myname",
-        age=25,
-        birth_date=datetime.date(2000, 1, 1),
-        observed=datetime.datetime(2000, 1, 1, 12, 30, 00),
-        meta={"yes": "no"}
+        myfield=example_value
     ))
     obs = repo.filter_by().all()
     assert obs == [
         MyItem(
             id="a",
-            name="myname",
-            age=25,
-            birth_date=datetime.date(2000, 1, 1),
-            observed=datetime.datetime(2000, 1, 1, 12, 30, 00),
-            meta={"yes": "no"}
+            myfield=example_value
         )
     ]
 
+@pytest.mark.parametrize("cls,nullable,sql_type", [
+    pytest.param(str, False, 'String', id="str"),
+    pytest.param(int, False, 'Integer', id="int"),
+    pytest.param(float, False, 'Float', id="float"),
+
+    pytest.param(datetime.date, False, 'Date', id="datetime.date"),
+    pytest.param(datetime.datetime, False, 'DateTime', id="datetime.datetime"),
+    pytest.param(datetime.timedelta, False, 'Interval', id="datetime.timedelta", marks=pytest.mark.skip("SQLite does not support Interval")),
+
+    pytest.param(dict, False, 'JSON', id="dict"),
+
+    pytest.param(typing.Optional[str], True, 'String', id="Optional[str]"),
+    pytest.param(typing.Union[str, None], True, 'String', id="Union[str, None]"),
+    pytest.param(typing.Union[None, str], True, 'String', id="Union[None, str]"),
+    pytest.param(Literal['yes', 'no'], False, 'String', id="Literal['...', '...']", marks=pytest.mark.skipif(sys.version_info < (3, 8), reason="<py38")),
+    pytest.param(Literal[1, 2], False, 'Integer', id="Literal[1, 2]", marks=pytest.mark.skipif(sys.version_info < (3, 8), reason="<py38")),
+    pytest.param(typing.Optional[Literal['yes', 'no']], True, 'String', id="Optional[Literal['...', '...']]", marks=pytest.mark.skipif(sys.version_info < (3, 8), reason="<py38")),
+])
+def test_init_column(cls, nullable, sql_type):
+    pytest.importorskip("sqlalchemy")
+
+    class MyItem(BaseModel):
+        __id_field__ = "id"
+        id: str
+        myfield: cls
+    import sqlalchemy
+    from sqlalchemy import create_engine
+    engine = create_engine('sqlite://')
+    repo = SQLRepo(model=MyItem, engine=engine, table="mytable", if_missing="create", id_field="id")
+    
+    # Test column
+    column = repo.model_orm.__table__.columns['myfield']
+    assert column.nullable is nullable
+    assert isinstance(column.type, getattr(sqlalchemy, sql_type))
+
+@pytest.mark.parametrize("cls", [
+    pytest.param(Literal[2, 'two'], id="Literal[2, 'two']", marks=pytest.mark.skipif(sys.version_info < (3, 8), reason="<py38")),
+    pytest.param(typing.Union[str, int], id="Union[str, int]", marks=pytest.mark.skipif(sys.version_info < (3, 8), reason="<py38")),
+    pytest.param(typing.Union[str, int, None], id="Union[str, int, None]", marks=pytest.mark.skipif(sys.version_info < (3, 8), reason="<py38")),
+])
+def test_init_column_fail(cls):
+    pytest.importorskip("sqlalchemy")
+
+    class MyItem(BaseModel):
+        __id_field__ = "id"
+        id: str
+        myfield: cls
+
+    from sqlalchemy import create_engine
+    engine = create_engine('sqlite://')
+
+    with pytest.raises(TypeError):
+        repo = SQLRepo(model=MyItem, engine=engine, table="mytable", if_missing="create", id_field="id")
+
 def test_init_create_table():
     pytest.importorskip("sqlalchemy")
     from sqlalchemy import create_engine
